import java.util.Random;
import java.util.ArrayList;

/**
 * Wumpus game player
 * 
 * @author Malcolm Ryan 
 * @version 20 Sep 2012
 */
public class Player
{
    private Map myMap;
    private Room myCurrentRoom;
    int myArrows;
    public static final int NUM_ARROWS = 3;
    
    /**
     * Create a new player in the given map.
     * The player's location is initially null.
     */
    public Player(Map map) {
        myMap = map;        
        myArrows = NUM_ARROWS;
        
        // initially nowhere
        myCurrentRoom = null;

    }

    /**
     * Print a list of neighbouring rooms and ask the player
     * to choose one.
     */
    private Room readNeighbouringRoom() {
        // get nearby rooms
        ArrayList<Room> neighbours = myCurrentRoom.getNeighbours();

        // repeat until the player makes a valid choice
        Room choice = null; 
        while (choice == null) {
        
            // print the list
            System.out.print("Choose [");
            for (int i = 0; i < neighbours.size(); i++) {
                if (i > 0) {
                    System.out.print(", ");
                }
                System.out.print(neighbours.get(i).getRoomID());
            }
            System.out.print("] ");
 
            // read an integer and check it matches one of the rooms
            Integer id = Input.readInt();
            if (id != null) {            
                for (Room r : neighbours) {                
                    if (r.getRoomID() == id) {
                        choice = r;
                    }
                }
            }        
        }
        
        // return the choice
        return choice;
    }

    /**
     * Move into a room, activating it.
     * 
     * @returns True if the game is over as a result (player dies/wins)
     */
    public boolean moveTo(Room room) {
        myCurrentRoom = room;
        return room.enter(this);
    }

    /**
     * Move into random room, activating it.
     * 
     * @returns True if the game is over as a result (player dies/wins)
     */
    public boolean moveRandom() {
        Room r = myMap.getRandomRoom();
        return moveTo(r);
    }
    
    /**
     * Shoot into a neighbouring room.
     * 
     * @returns True if the game is over (killed wumpus or ran out of arrows)
     */
    private boolean shoot(Room room) {
        // shoot the room and see if this ended the game
        boolean gameOver = room.shoot();
           
        // use up one arrow
        myArrows--;
        if (!gameOver && myArrows == 0) {
            // if we're out of arrows, the game is over
            System.out.println("You are out of arrows!");
            gameOver = true;
        }
        return gameOver;
    }
    
    /**
     * Play one turn of the game. Move or shoot.
     * 
     * @returns True if the game is over.
     */
    private boolean playTurn() {
        
        boolean gameOver = false;
        
        // prompt the player and read a character
        System.out.print("(M)ove or (S)hoot? ");
        char c = Input.readChar();
        
        if (c == 'M') {
            // move
            System.out.println("Which room do you want to move to?");
            Room room = readNeighbouringRoom();
            gameOver = moveTo(room);
        }
        else if (c == 'S') {
            // shoot
            System.out.println("Which room do you want to shoot to?");
            Room room = readNeighbouringRoom();
            gameOver = shoot(room);
        }
        
        return gameOver;
    }
    
    /**
     * Play the game until it is over
     */
    private void play() {
        System.out.println("Welcome to Wumpus World!");
        System.out.println();
        
        // enter a random room to start
        boolean gameOver = moveRandom();
        
        while (!gameOver) {
            gameOver = playTurn();            
        }
        
        System.out.println("GAME OVER!");
    }
    
    /**
     * Main method. 
     */
    public static void main(String args[]) {
        // create a map and a player
        Map map = new Map();
        Player player = new Player(map);
        
        // start the game
        player.play();
    }
    
}
