import java.util.ArrayList;
import java.util.List;

/**
 * A room in the Wumpus map
 * 
 * @author Malcolm Ryan
 * @version 20 Sep 2012
 */
public class Room 
{
    private int myRoomID;
    private ArrayList<Room> myNeighbours;    
    
    private List<Hazard> myHazards;
    
    /**
     * Create a room with the given id.
     * Initially it had no neighbours and no hazards.
     */
    public Room(int id) {
        myRoomID = id; 
        myNeighbours = new ArrayList<Room>();
        myHazards = new ArrayList<Hazard>();
    }
    
    /**
     * Get the room's id in the map
     */
    public int getRoomID() {
        return myRoomID;
    }

    /**
     * Get the neighbouring rooms
     */
    public ArrayList<Room> getNeighbours() {
        return myNeighbours;
    }
    
    /**
     * Connect the room to a neighbouring room
     */
    public void connectTo(Room room) {
        myNeighbours.add(room);
    }    
    
    /**
     * Add a hazard to the room
     */
    public void addHazard(Hazard h) {
        myHazards.add(h);
    }
    
    /**
     * Show any warnings for hazards this room
     */
    public void showWarnings() {
        for (Hazard h : myHazards) {
            String warning = h.getWarning();
            System.out.println(warning);
        }
    }

    /**
     * Show any warnings for hazards in neighbouring rooms
     */
    public void showNearbyWarnings() {
        for (Room r : myNeighbours) {
            r.showWarnings();
        }
    }

    
    /**
     * Activate any hazards in the room
     */
    public boolean activateHazards(Player p) {
        for (Hazard h : myHazards) {
            boolean gameOver = h.activate(p);
            if (gameOver) {
                return true;
            }
        }
        return false;
    }
       
    /**
     * The player has entered the room. 
     * Show a description then activate the hazards
     */
    public boolean enter(Player player) {
        // describe the room
        System.out.println("You are in Room " + myRoomID);
        System.out.print("Exits lead to rooms");
        
        for (Room r : myNeighbours) {
            System.out.print(" " + r.getRoomID());
        }
        System.out.println();
        
        // show any warnings for nearby hazards
        showNearbyWarnings();
        
        // activate and hazards in the room
        boolean gameOver = activateHazards(player);        
        return gameOver;
    }
    
    /**
     * Handle the outcomes if the player shoots into this room
     */
    public boolean shoot() {
        
        // Try shooting any of the hazards
        // stop if gameOver.
        
        for (Hazard h : myHazards) {
            boolean gameOver = h.shoot();
            if (gameOver) {
                return true; // GAME OVER
            }
        }

        // Nothing happened
        System.out.println("Your arrow falls with a clatter to the floor!");
        return false;
        
    }
}
